import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { ErrorResponse } from 'packages/commons/service/error-response';

import { Infra } from '../../model';
import { InfraService } from '../../service/infra.service';

import {
  Readbydomain,
  ReadbydomainSuccess,
  ReadbydomainFailure,
  ActionType,
} from './readbydomain.action';
import { Domain } from 'packages/domain/model';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private infraService: InfraService,
    private router: Router
  ) { }

  @Effect()
  readbydomain$: Observable<Action> = this.actions$
    .ofType(ActionType.Readbydomain)
    .map((action: Readbydomain) => action.payload)
    .exhaustMap(member =>
      this.infraService
        .readByDomain(member)
        .map(_member => new ReadbydomainSuccess(_member))
        .catch(error => of(new ReadbydomainFailure(error)))
    );

  @Effect({ dispatch: false })
  readbydomainSuccess$ = this.actions$
    .ofType(ActionType.ReadbydomainSuccess)
    .do(() => this.router.navigate(['/']));

}
