import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Store, select } from '@ngrx/store';

import { ErrorResponse } from 'packages/commons/service/error-response';

import * as AuthStore from '../../store/auth';
import { AuthSelector } from '../../store';

@Component({
  selector: 'of-member-signin',
  templateUrl: './signin.component.html',
  styleUrls: ['./signin.component.scss']
})
export class SigninComponent implements OnInit {
  pending$ = this.store.pipe(select(AuthSelector.select('pending')));
  error$ = this.store.pipe(select(AuthSelector.select('error')));

  errorMessage: string | null;

  signinForm: FormGroup;
  formErrors = {
    'email': '',
    'password': ''
  };

  constructor(
    private router: Router,
    private store: Store<AuthStore.State>,
    private formBuilder: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.initForm();

    this.pending$.subscribe((pending: boolean) => {
      if (pending) {
        this.signinForm.disable();
      } else {
        this.signinForm.enable();
      }
    });

    this.error$.subscribe((error) => {
      if (error) {
        this.errorMessage = error.exception;
      } else {
        this.errorMessage = null;
      }
    });
  }

  initForm() {
    this.signinForm = this.formBuilder.group({
      'email': [
        '',
        [
          Validators.required,
          Validators.email
        ]
      ],
      'password': [
        '',
        [
          Validators.pattern('^(?=.*[0-9])(?=.*[a-zA-Z])([a-zA-Z0-9]+)$'),
          Validators.minLength(6),
          Validators.maxLength(25)
        ]
      ],
    });

  }

  resetPasswordBtnClick() {
    this.router.navigateByUrl('/auth/reset-password');
  }

  signupBtnClick() {
    this.router.navigateByUrl('/auth/signup');
  }
  signin() {
    this.store.dispatch(new AuthStore.Signin(this.signinForm.value));
    console.log('signin component signin fnc');
  }
}
